#!{{ python_path }}
import time
import xmltodict
import sys
sys.path.append({{ repr(buildout_directory_path) }})
from ncclient_common import LopcommNetconfClient

if __name__ == '__main__':
  nc = LopcommNetconfClient(
    log_file="{{ log_file }}",
    software_reply_json_log_file="{{ software_reply_json_log_file }}"
    )
  while True:
      try:
          nc.connect("fe80::20a:ff:fe00:1020%slaptap6", 830, "oranuser", "oranpassword")
          # Fetch software inventory

          inventory_vars = nc.get_inventory()

          nonrunning_slot_name = inventory_vars["nonrunning_slot_name"]
          running_slot_name = inventory_vars["running_slot_name"]
          active_nonrunning_slot_name = inventory_vars["active_nonrunning_slot_name"]
          nonrunning_slot_name_build_version = inventory_vars["nonrunning_slot_name_build_version"]
          running_slot_name_build_version = inventory_vars["running_slot_name_build_version"]

          if running_slot_name and nonrunning_slot_name:
              if running_slot_name:
                  nc.logger.info("One slot is running and one is non-running. Proceeding...")
                  if running_slot_name_build_version in "{{firmware_name}}":
                      nc.logger.info("Running slot's build-version %s is already updated. Skipping install." % running_slot_name_build_version)
                  else:
                      nc.logger.info("Current build version: %s" % running_slot_name_build_version)
                      download_rpc_xml = f"""
                          <software-download xmlns="urn:o-ran:software-management:1.0">
                              <remote-file-path>{{remote_file_path}}/{{firmware_name}}</remote-file-path>
                          </software-download>
                      """
                      download_reply_xml = nc.custom_rpc_request(download_rpc_xml)
                      if download_reply_xml:
                          download_data = xmltodict.parse(download_reply_xml)
                          nc.software_reply_json_logger.info('', extra={'data': download_data})

                      install_rpc_xml = f"""
                          <software-install xmlns="urn:o-ran:software-management:1.0">
                              <slot-name>{nonrunning_slot_name}</slot-name>
                              <file-names>{{firmware_name}}</file-names>
                          </software-install>
                      """
                      install_reply_xml = nc.custom_rpc_request(install_rpc_xml)
                      if install_reply_xml:
                          install_data = xmltodict.parse(install_reply_xml)
                          nc.software_reply_json_logger.info('', extra={'data': install_data})

                      activate_rpc_xml = f"""
                          <software-activate xmlns="urn:o-ran:software-management:1.0">
                              <slot-name>{nonrunning_slot_name}</slot-name>
                          </software-activate>
                      """
                      activate_reply_xml = nc.custom_rpc_request(activate_rpc_xml)
                      if activate_reply_xml:
                          activate_data = xmltodict.parse(activate_reply_xml)
                          nc.software_reply_json_logger.info('', extra={'data': activate_data})

                      nc.get_inventory()
                      if nonrunning_slot_name_build_version in "{{firmware_name}}" and active_nonrunning_slot_name:
                          nc.logger.info("Active non-running slot has the updated build version. Resetting device.")
                          nc.reset_device()
          break
      except Exception as e:
          nc.logger.debug('Got exception, waiting 10 seconds before reconnecting...')
          nc.logger.debug(str(e))
          time.sleep(10)
      finally:
          nc.close()
